/*  JAL - Cross-platform Java Application Loader for Motorola mobile phones
 *  Copyright (C) 2006  Andrew Ivanov <andrew@softmile.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>
#include "config.h"
#include "errors.h"


const char* program_name;
char percent_buf[5];
int debug = 0;

BAUD_RATE get_baudrate(char *baudrate_string){
	int i;
	for (i=0; i<18; i++) {
		if (!strcasecmp(baudrates_strings[i], baudrate_string)) {
			printf("%s\n", baudrates_strings[i]);
			return dwBaudRates[i];
		}
	}
	return 0;
}

char *get_percent(char *percent_buf, int percentage){
	sprintf(percent_buf, "%3d%s", percentage ,"%");
	return percent_buf;
}

void print_usage ( FILE* stream, int exit_code )
{
	fprintf(stream, "%s\n", DESCRIPTION);
	fprintf(stream, "Version: %s\n", VERSION);
	fprintf(stream, "Usage: %s options [ inputfile ]\n", program_name);
	fprintf(stream, 
	"	-h --help		Display this usage information\n"
	"	-d --device device	Specify phone device\n"
	"	-b --baud baud		Specify baud (B300, B600, B1200, B2400, B4800, B9600, B19200, B38400, B57600, B115200)\n"
	"	-p --ports		Show available COM ports (Windows only)\n"
	"	-D --debug		Debugging\n"
	);
	exit(exit_code);
}

void print_error_and_exit(char *error_message, int exit_code){
	fprintf(stderr, "%s\n", error_message);
	exit(exit_code);
}

void print_available_ports()
{
#ifdef WIN32
	int i;
	for (i = 1; i <= 32; i++) {
		char port[6] = "COM00";
		itoa(i, port+3, 10);
		HANDLE handle = CreateFileA(port, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0);
		if ( handle != INVALID_HANDLE_VALUE ) {
			fprintf(stdout, "%s\n", port);
			CloseHandle(handle);
		}
	}
	exit(0);
#else
	print_error_and_exit("This function works in Windows only", NOT_IMPLEMENTED);
#endif
}

void _info_cb ( const char *info )
{
	fprintf(stdout, "%s\n", info);
}

void _progress_cb ( int pos )
{
	if (pos < 0){
		return;
	}
	fprintf(stdout, "\b\b\b\b");
	fprintf(stdout, "%s", get_percent(percent_buf, pos));
	if (pos == 100)
	{
		fprintf(stdout, "\n");
	}
}

int main( int argc, char* argv[] )
{
	const char* const short_options="hpd:b:D";

	const struct option long_options[] = {
		{ "help",	0,	NULL,	'h' },
		{ "ports",	0,	NULL,	'p' },
		{ "device",	1,	NULL,	'd' },
		{ "baud",	1,	NULL,	'b' },
		{ "debug",	0,	NULL,	'D' },	
		{  NULL,	0,	NULL,	 0  }
	};
	
	const char* phone_device	= NULL;
	const char* input_file	= NULL;
	int next_option;
	
	BAUD_RATE baud_rate;
	baud_rate = dwBaudRates[6];

	program_name = basename(argv[0]);

	setvbuf(stdout, NULL, _IONBF, 0);

	do {
		next_option = getopt_long (argc, argv, short_options, long_options, NULL);
		switch ( next_option ) {
			case 'h':
				print_usage (stdout, 0);
				break;
			case 'p':
				print_available_ports();
				break;
			case 'd':
				phone_device = optarg;
				break;
			case 'b':
				baud_rate = get_baudrate(optarg);
				if (!baud_rate) {
					print_error_and_exit(INVALID_BAUDRATE_MSG, INVALID_BAUDRATE);
				}
				break;
			case 'D':
				debug = 1;
				break;
			case '?':
				print_usage (stderr, 1);
			case -1:
				break;
			default:
				abort();
		}
	} while ( next_option!=-1 );
	
	jal_set_info_callback( _info_cb );

	if ( debug ) {	
		jal_debug( phone_device, baud_rate );
	} else {
		jal_set_progress_callback( _progress_cb );
		if ( optind+1==argc ) {
			input_file=argv[optind];
		} else {
			print_usage (stderr, 1);
		}
		printf("JAD file:	%s\n", input_file);
		printf("Phone device:	%s\n", phone_device);
		
		if ( !phone_device ) {
			print_error_and_exit(NO_PHONE_DEVICE_MSG, NO_PHONE_DEVICE);
		}

		jal_set_speed( SPEED );
		jal_upload_file( phone_device, baud_rate, input_file );
	}
	return 0;	
}
