/*  JAL - Cross-platform Java Application Loader for Motorola mobile phones
 *  Copyright (C) 2006  Andrew Ivanov <andrew@softmile.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include "jal_config.h"
#include "libjal.h"

static cb_info_fn	info_fn = NULL;
static cb_progress_fn	progress_fn = NULL;

static int delay = 110000;

static HANDLE phone_handle = INVALID_HANDLE_VALUE;

#ifdef WIN32

HANDLE open( const char *file_name, DWORD dwDesiredAccess)
{
	return CreateFileA( file_name, dwDesiredAccess, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
}

int read (HANDLE handle, char *buf, size_t size)
{
	DWORD lpNumberOfBytesRead = 0;
	if ( ReadFile(handle, buf, size, &lpNumberOfBytesRead, NULL) ) {
		return lpNumberOfBytesRead;
	} else {
		return -1;
	}
}

int write (HANDLE handle, const char *buf, size_t size)
{
	DWORD lpNumberOfBytesWritten;
	if ( WriteFile(handle, buf, size, &lpNumberOfBytesWritten, NULL) ) {
		return lpNumberOfBytesWritten;
	} else {
		return -1;
	}	
}

void close( HANDLE handle )
{
	CloseHandle( handle );
}

void usleep( int interval )
{
	Sleep( interval/1000 );
}

#endif


void call_info_cb( const char *ch_info )
{
	if ( info_fn ) {
		info_fn( ch_info );
	}
}

void call_progress_cb( int pos )
{
	if ( progress_fn ) {
		progress_fn( pos );
	}
}


HANDLE open_phone_device ( const char* phone_device, const BAUD_RATE speed )
{
#ifndef WIN32
	struct termios options;
	int fd;
	
	fd = open(phone_device, O_RDWR | O_NOCTTY | O_NDELAY);
	if ( fd==-1 ) {
		return -1;
	}
	fcntl(fd, F_SETFL, 0);
 	tcgetattr(fd, &options);
	cfmakeraw(&options);
	cfsetispeed(&options, speed);
	cfsetospeed(&options, speed);
	
	options.c_cc[VMIN]	=  0;
	options.c_cc[VTIME]	=  10;

	if (tcsetattr(fd, TCSANOW, &options)) {
		return -1;
	}
	tcflush(fd, TCIOFLUSH);
	return fd;
#else
	HANDLE handle = CreateFileA(phone_device, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
	if ( handle != INVALID_HANDLE_VALUE ) {
		DCB dcb;
		COMMTIMEOUTS tcm;
		
		dcb.BaudRate = speed;
		dcb.ByteSize = 8;
		dcb.Parity = NOPARITY;
		dcb.fRtsControl=RTS_CONTROL_ENABLE;
		dcb.fDtrControl=DTR_CONTROL_ENABLE;
		dcb.StopBits = ONESTOPBIT;
		SetCommState(handle, &dcb);
		
		tcm.ReadIntervalTimeout = 0;
		tcm.ReadTotalTimeoutMultiplier = 0; 
		tcm.ReadTotalTimeoutConstant = 1000;
		tcm.WriteTotalTimeoutMultiplier = 0;
		tcm.WriteTotalTimeoutConstant = 0;
		SetCommTimeouts( handle, &tcm );
	}
	return handle;
#endif
}


size_t get_file_size(const char* file_name) {
	struct stat file_stat;
	stat(file_name, &file_stat);
	return file_stat.st_size;
}


int send_data ( HANDLE phone_fd, char *buf, int length ) {
	return write(phone_fd, buf, length);
}

int read_reply( HANDLE phone_fd, char *buf, int length ) {
	return read(phone_fd, buf, length);
}



int get_standard_reply( HANDLE phone_fd ){
	char reply[STRING_LENGTH];
	int res;
	res=read_reply(phone_fd, reply, 2);
	if (res!=2) {
		call_info_cb ( "Warning: Incomplete reply received" );
		return 0;//-1;
	} else {
#ifdef EXACT_REPLY
		if ( memcmp(reply, good_reply, 2) ){
			call_info_cb("Incorrect reply received");
			return -1;
		}
#endif
	}
	return 0;
}


int send_file ( HANDLE phone_fd, const char *file_name, int data_end_type ) {
	int res;
	char buf[STRING_LENGTH];
	int result=0;
	int counter;
	size_t file_size;
	int percentage = 0;
	char p_start[2];
	memcpy(p_start, packet_start, 2);
	
	HANDLE input_file_fd = open(file_name, O_RDONLY);
	if ( input_file_fd == INVALID_HANDLE_VALUE ) {
		sprintf( buf, "Can't open file %s" ,file_name );
		call_info_cb( buf );
		return -1;
	}
	file_size=get_file_size(file_name);
	counter=0;
	call_progress_cb( percentage );
	do{
		res=read(input_file_fd, buf, PACKET_SIZE);
		if ( res>0 ) {
			p_start[1]=res;
			write( phone_fd, p_start,	2   );
			write( phone_fd, buf,		res );
#ifndef WIN32
			tcflush(phone_fd, TCIOFLUSH);
#endif
			result=get_standard_reply( phone_fd );
			counter++;
			percentage = ((counter-1)*PACKET_SIZE+res)*100/file_size;
			call_progress_cb( percentage );
			usleep( delay );
			if (result) break;
		}
	} while (res>0);

	if (data_end_type==1) {
		write( phone_fd, data_end1,	2 );
	} else {
		write( phone_fd, data_end2,	2 );
	}
	
	close(input_file_fd);
	return result;
}


int send_jad ( HANDLE phone_fd, JAD_INFO *jad_info ) {
	char header[6];
	
	memcpy(header, jad_header, 6);
	
	header[4] = jad_info->jad_size >> 8;
	header[5] = jad_info->jad_size;
	
	send_data(phone_fd, header, 6);
	
	if (get_standard_reply( phone_fd )) {
		return -1;
	}
	return send_file ( phone_fd, jad_info->jad_file, 1 );
}



int get_jad_info( const char *jad_file_name, JAD_INFO *jad_info ) {
	struct stat jad_stat;
	FILE *f;
	char buf[STRING_LENGTH];
	char copy[STRING_LENGTH];
	char *delims=": \t\r\n";
	
	if ( stat (jad_file_name, &jad_stat) ) {
		return -1;
	}
	jad_info->jad_size = jad_stat.st_size;
	f=fopen(jad_file_name, "r");
	if ( !f ) {
		return -1;
	}
	
	strcpy( jad_info->jad_file, jad_file_name );
	memset( jad_info->jar_file, 0, STRING_LENGTH );
	while ( fgets( buf, STRING_LENGTH, f ) ){
		strcpy(copy, buf);
		if ( !strcasecmp( (char *)strtok(buf, delims), "MIDlet-Jar-URL") ){			
			char *file_name = strchr(copy, ':') + 1;
			while ( *file_name == ' ' || *file_name == '\t' )
			{
				file_name++;
			}
			char *cr = strchr(file_name, '\n');
			if (cr) *cr = 0;
			char *lf = strchr(file_name, '\r');
			if (lf) *lf = 0;
			strcpy( jad_info->jar_file, file_name);
			break;
		}
	}
	if ( !jad_info->jar_file[0] ) {
		return -1;
	} else {
		return 0;
	}
	
	
}


int get_jad_security_reply ( HANDLE phone_fd ) {
	char last2[2];
	int res;
	int sign;
	
	
	if (get_standard_reply( phone_fd ))
		return -1;
		
	last2[0]=0xFF;
	last2[1]=0xFF;
	sign=0;
	call_info_cb( "JAD file sent. Waiting for reply..." );
	while (1) {
		last2[0]=last2[1];
		res=read_reply(phone_fd, &last2[1], 1);
		if ( res==0 && sign ) {
			break;
		}
		if (res>0){
			sign=0;
			if ( last2[0]==0x00 && last2[1]==0x03 ) {
				call_info_cb( ">>>OK" );
				return 0;
			}
			
			if ( last2[0]=='.' && last2[1]==0x0A ) {
				sign=1;
			}			
			
		}
	}
	call_info_cb( ">>>CANCEL" );
	return -1;
}


int send_jar ( HANDLE phone_fd, JAD_INFO *jad_info ) {
	char last2[2];
	int res;
	int sign=0;
	char jar_file_full_path[STRING_LENGTH];
	char header[2];
	
	memcpy(header, jar_header, 2);
	header[1]=strlen(jad_info->jar_file);

#ifdef WIN32
	char drive[STRING_LENGTH];
	char dir[STRING_LENGTH];
	_splitpath( jad_info->jad_file, drive, dir, NULL, NULL );
	if (!strlen(drive) && !strlen(dir))
	{
		strcpy(dir, "./");
	}
	snprintf(jar_file_full_path, STRING_LENGTH, "%s%s%s", drive, dir, jad_info->jar_file);
#else
	snprintf(jar_file_full_path, STRING_LENGTH, "%s/%s", dirname(jad_info->jad_file), jad_info->jar_file);
#endif
	send_data(phone_fd, header, 2);
	send_data(phone_fd, jad_info->jar_file, strlen(jad_info->jar_file));
	
	while (1) {
		last2[0]=last2[1];
		res=read_reply(phone_fd, &last2[1], 1);

		if ( res==0 && sign ) {
			return -1;
		}
		if (res>0){
			sign=0;
			if ( last2[0]==0x00 && last2[1]==0x01 ) {
				break;
			}
			
			if ( last2[0]=='.' && last2[1]==0x0A ) {
				sign=1;
			}			
			
		}
	}

	return send_file( phone_fd, jar_file_full_path, 2 );
}


void clear_buffer ( HANDLE phone_fd ) {
	int res;
	char byte;
	static char buf[1024];
	static int pos = 0;
	while ( 1 ) {
		res=read_reply(phone_fd, &byte, 1);
		if (res > 0) {
			if ( byte == 10 ) {
				if (pos)
				{
					if (buf[pos - 1] == 13)
					{
						pos--;
					}
					buf[pos] = 0;
					pos = 0;
					call_info_cb ( buf );
				}				
			} else {
				buf[pos] = byte;
				pos++;
			}
		} else {
			break;
		}
	}
}


void jal_upload_file( const char *port, BAUD_RATE baud, const char *file_name)
{
	JAD_INFO jad_info;
	if ( get_jad_info( file_name, &jad_info ) ) {
		return;
	}
	HANDLE phone_fd=open_phone_device( port, baud );
	call_progress_cb( -1 );
	if ( phone_fd == INVALID_HANDLE_VALUE ) {
		call_info_cb ( "Error opening phone device" );
	} else {
		phone_handle = phone_fd;
		call_info_cb ( "Phone device opened" );
		call_info_cb ( "Flushing buffers" );
		clear_buffer( phone_fd );
		call_info_cb ( "Sending JAD file..." );
		if ( send_jad ( phone_fd, &jad_info ) == -1 ) {
			call_info_cb ( "Error sending JAD file" );
		} else {
			if ( get_jad_security_reply(phone_fd) == -1 ) {
				// user has cancelled operation
			} else {
				call_info_cb ( "Sending JAR file..." );
				if ( send_jar(phone_fd, &jad_info) == -1 ) {
					call_info_cb ( "Error sending JAR file" );
				} else {
					call_info_cb ( "Application uploaded" );
				}
			}
		}
		call_info_cb ( "Flushing buffers" );
		usleep(3000000);
		clear_buffer( phone_fd );
		call_info_cb ( "Closing phone device"  );
		close(phone_fd);
		call_info_cb ( "DONE" );
		phone_handle = INVALID_HANDLE_VALUE;
	}
	call_progress_cb( -1 );
}


void jal_set_info_callback( cb_info_fn cb )
{
	info_fn = cb;
}

void jal_set_progress_callback( cb_progress_fn cb )
{
	progress_fn = cb;
}

void jal_set_speed ( int speed )
{
	delay = ( 90 - ( speed - 50 ) / 3 ) * 1000;
}

void jal_close_device()
{
	if ( phone_handle != INVALID_HANDLE_VALUE ) {
		close( phone_handle );
	}
}


void jal_debug( const char *port, BAUD_RATE baud )
{
	HANDLE phone_fd=open_phone_device( port, baud );
	call_progress_cb( 0 );
	if ( phone_fd == INVALID_HANDLE_VALUE ) {
		call_info_cb ( "Error opening phone device" );
	} else {
		phone_handle = phone_fd;
		call_info_cb ( "Phone device opened" );
		call_info_cb ( "Debugging started" );
		while (1) {
			clear_buffer(phone_handle);
		}
	}
}

