/*  JAL - Cross-platform Java Application Loader for Motorola mobile phones
 *  Copyright (C) 2006  Andrew Ivanov <andrew@softmile.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <QFileDialog>
#include <QTextStream>
#include <QSettings>
#include <QMessageBox>
#include <QProgressBar>

#ifndef WIN32
#include <libgen.h>
#endif

#include "main_wnd.h"
#include "settings.h"
#include "about.h"

MainWnd::MainWnd( QWidget * parent, Qt::WFlags flags )
    : QMainWindow( parent, flags ),
	  information( tr("Log"), this, Qt::WFlags(Qt::BottomDockWidgetArea) )
{
    ui.setupUi(this);
	log.setReadOnly(true);
    information.setWidget(&log);
	information.setAllowedAreas( Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea );
	addDockWidget(Qt::BottomDockWidgetArea, &information);
	
	
	QSize size( 300, 250 );
	ui.textEditJAD->setMinimumSize( size );

	connect(ui.actionOpen, SIGNAL(triggered()), this, SLOT(openFile()));
	connect(ui.actionSettings, SIGNAL(triggered()), this, SLOT(settings()));
	connect(ui.actionUpload, SIGNAL(triggered()), this, SLOT(upload()));
	connect(ui.actionCancel_upload, SIGNAL(triggered()), this, SLOT(cancel_upload()));
	connect(ui.actionDebug, SIGNAL(toggled(bool)), this, SLOT(debug(bool)));
	
	connect(ui.actionQuit, SIGNAL(triggered()), this, SLOT(quit()));
	connect(ui.actionClear_log, SIGNAL(triggered()), this, SLOT(clearLog()));
	connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(about()));
	
	connect(&m_conn, SIGNAL(info(QString)), this, SLOT(info(QString)));
	connect(&m_conn, SIGNAL(progress(int)), this, SLOT(progress(int)));
	
	connect(&m_conn, SIGNAL(started()), this, SLOT(upload_started()));
	connect(&m_conn, SIGNAL(finished()), this, SLOT(upload_finished()));
	connect(&m_conn, SIGNAL(terminated()), this, SLOT(upload_terminated()));
	
	ui.menuView->insertAction(ui.actionClear_log, ui.toolBar->toggleViewAction());
	ui.menuView->insertAction(ui.actionClear_log, information.toggleViewAction());
	ui.menuView->insertSeparator(ui.actionClear_log);
	
		
	m_conn.setParent( this );
		
	ui.actionUpload->setEnabled( false );
	ui.actionCancel_upload->setEnabled( false );

	
}


void MainWnd::openFile()
{
	QSettings settings;
	QString sFileName = QFileDialog::getOpenFileName(
		this,
		tr("Choose a file"),
		settings.value( "last_dir" ).toString( ),
		"JAD (*.jad)");
	if ( !sFileName.isNull() ) {
		loadJAD(sFileName);
		char dir[512];
		char path[512];
#ifdef WIN32
		char drive[16];
		
		_splitpath( sFileName.toLocal8Bit( ), drive, dir, NULL, NULL );
		snprintf( path, 512, "%s%s", drive, dir );
#else
		qstrcpy( dir,  sFileName.toLocal8Bit( ) );		
		snprintf(path, 512, "%s", dirname( dir ));
#endif
		settings.setValue( "last_dir", path );
	}
}

void MainWnd::settings()
{
	Settings dlg(this);
	dlg.exec();
}

void MainWnd::clearLog()
{
	log.clear();
}

void MainWnd::upload()
{
	QSettings settings;
	
	if ( !m_conn.isRunning() ) {
		m_conn.init( settings.value( "port" ).toString( ), settings.value( "baud", 6 ).toInt( ), settings.value( "speed", 50 ).toInt( ), m_sJADFileName );
		m_conn.upload();
	}

}

void MainWnd::cancel_upload()
{
	if ( m_conn.isRunning() ) {
		if ( QMessageBox::critical( this, tr( "Cancel upload" ), tr( "It's not recommended to use this because it can damage you phone! Continue?" ), tr("&Yes"), tr("&No"), QString(), 0, 1) )  {
			return;
		}
		m_conn.terminate();
		m_conn.wait();
		jal_close_device();
		info( tr( "Upload cancelled" ) );
		progress( 0 );
	}
}

void MainWnd::debug( bool value )
{
	QSettings settings;	
	if ( value ) {
		if ( !m_conn.isRunning() ) {
			m_conn.init( settings.value( "port" ).toString( ), settings.value( "baud", 6 ).toInt( ), settings.value( "speed", 50 ).toInt( ), m_sJADFileName );
			m_conn.debug();
		}
	} else {
		m_conn.terminate();
		m_conn.wait();
		jal_close_device();
		info( tr( "Debugging stopped" ) );
	}
}

void MainWnd::loadJAD(QString sFileName)
{
	m_sJADFileName = sFileName;
	QFile file( m_sJADFileName );
	
	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		return;
	
	ui.textEditJAD->clear();
	QTextStream in(&file);
	while (!in.atEnd()) {
		QString line = in.readLine();
		ui.textEditJAD->append(line);
	}
	QString shownName = QFileInfo(m_sJADFileName).fileName(); 
	setWindowTitle(tr("%1 - %2").arg(shownName).arg(tr("Java Application Loader")));
	ui.actionUpload->setEnabled( true );
}

void MainWnd::quit()
{
	close();
}



void MainWnd::info( QString sInfo )
{
	log.append( sInfo );
	log.moveCursor(QTextCursor::End);
	log.ensureCursorVisible();
}

void MainWnd::progress( int pos )
{
	ui.progressBar->setValue( pos );
}


void MainWnd::upload_started()
{
	m_bUploadActionState = ui.actionUpload->isEnabled();
	ui.actionUpload->setEnabled( false );
	if ( m_conn.isDebug() ) {
	} else {
		ui.actionDebug->setEnabled( false );
		ui.actionCancel_upload->setEnabled( true );
	}
}

void MainWnd::upload_finished()
{
	if ( m_conn.isDebug() ) {
		ui.actionDebug->setChecked( false );
		ui.actionUpload->setEnabled( m_bUploadActionState );
	} else {
		ui.actionUpload->setEnabled( true );
		ui.actionDebug->setEnabled( true );
		ui.actionCancel_upload->setEnabled( false );
	}
}

void MainWnd::upload_terminated()
{
	upload_finished();
}

void MainWnd::about()
{
	About aboutDlg;
	aboutDlg.exec();
}
